/**
 * TSVReaderTokenizer.java
 */
package UTEStatsViewer;
import java.io.*;
/**
 * Reads a Tab Separated Value Input Stream as a stream of values.
 */
public class TSVReaderTokenizer {
   /** special value returned by nextToken() to indicate end of file */
   public static final StringBuffer eofToken = new StringBuffer("");

   /** special value return by nextToken to indicated end of row */
   public static final StringBuffer eorToken = new StringBuffer("\n");

   /** special value returned by nextToken to indicate table separator
       character */
   public static final StringBuffer tableSepToken = new StringBuffer("\u000c");

   private Reader reader;

   private StringBuffer buf = new StringBuffer();

   private int prevChar = 0;

   private StringBuffer nextToken = null;


   /** Construct a Tab Separated Value stream. */
   public TSVReaderTokenizer(Reader reader) {
      this.reader = reader;
   }


   /**
    * Return the next token.  This method overwrites the buffer returned
    * by its previous invocation; the calling method must copy the
    * value returned by the previous invocation if it is needed.  Returns
    * end-of-file, end-of-row, and table-separator as special tokens.
    */
   public StringBuffer nextToken() throws IOException {
      // return special value, if need to
      if (nextToken != null) {
         StringBuffer rc = nextToken;
         nextToken = null;
         return rc;
      }

      // read first character of next token
      int ch = reader.read();

      // consume other character of CR-NL pair
      if ((ch == '\n') && (prevChar == '\r')) {
         // skip NL that immediately follows a CR
         ch = reader.read();
      } else if ((ch == '\r') && (prevChar == '\n')) {
         // skip CR that immediately follows a NL
         ch = reader.read();
      }


      /* read characters making up next token into buffer */
      buf.setLength(0);
      boolean appendedChar = false;
      readloop: while (true) {
         switch (ch) {
            case '\t': break readloop;
            case '\r':
            case '\n': nextToken = eorToken; break readloop;
            case 0x0c: nextToken = tableSepToken; break readloop;
            case -1:   nextToken = eofToken; break readloop;
            default:
               buf.append((char)ch);
               appendedChar = true;
               ch = reader.read();
         }
      }
      prevChar = ch;

      if (!appendedChar && (nextToken != null) ) {
         // no "real" token before a "special" token, return special token
         StringBuffer ret;
         ret = nextToken;
         nextToken = null;
         return ret;
      }

      return buf;
   }
}
